package family;
import java.sql.*;
import java.util.*;

public class Dao {

	private final String jdbcURI;

	public Dao( String jdbcURI ){
		if( jdbcURI == null ){
			throw new IllegalArgumentException("Null value not allowed.");
		}
		this.jdbcURI = jdbcURI;
	}

	public static class Exception extends RuntimeException {
		public Exception(){}
		public Exception(String message){ super(message); }
	}

	public void createTable(){
		try {
			Connection c = DriverManager.getConnection(jdbcURI);
			try {
					createTable(c);
			} finally { c.close(); }
		} catch( SQLException e ){
			throw new Dao.Exception(e.getMessage());
		}
	}

	public Set<Person> findByName( String name ){
		try {
			Connection c = DriverManager.getConnection(jdbcURI);
			try {
					return findByName(name,c);
			} finally { c.close(); }
		} catch( SQLException e ){
			throw new Dao.Exception(e.getMessage());
		}
	}

	public boolean persist( Person p ){
		try {
			Connection c = DriverManager.getConnection(jdbcURI);
			try {
					c.setAutoCommit(false);
					try {
						boolean success = persist(p,c);
						c.commit();
						return success;
					} catch( SQLException e ){
						c.rollback();
						throw e;
					}
			} finally { c.close(); }
		} catch( SQLException e ){
			throw new Dao.Exception(e.getMessage());
		}
	}

	private static final String createTable =
		"CREATE TABLE Person (" +
		" id INTEGER PRIMARY KEY," +
		" name VARCHAR(256) NOT NULL," +
		" ismale BIT NOT NULL," +
		" birth INTEGER NOT NULL," +
		" death INTEGER" +
		" )";

	private void createTable( Connection c ) throws SQLException {
		Statement s = c.createStatement();
		try {
			s.executeUpdate(createTable);
		} finally {
			s.close();
		}
	}

	private static final String insert =
		"INSERT INTO Person (name,ismale,birth,death) VALUES (?,?,?,?)";

	private boolean persist( Person p, Connection c ) throws SQLException {
		if( ! persisted(p,c) ){
			PreparedStatement s = c.prepareStatement(insert);
			try {
				s.setString(1,p.name);
				s.setBoolean(2,p.isMale);
				s.setInt(3,p.birth);
				s.setObject(4,p.death,Types.INTEGER);
				s.executeUpdate();
				return true;
			} finally { s.close(); }
		} else {
			return false;
		}
	}

	private static final String select =
		"SELECT death FROM Person WHERE name=? AND ismale=? AND birth=?";

	private boolean persisted( Person p, Connection c ) throws SQLException {
		PreparedStatement s = c.prepareStatement(select);
		try {
			s.setString(1,p.name);
			s.setBoolean(2,p.isMale);
			s.setInt(3,p.birth);
			ResultSet rs = s.executeQuery();
			while( rs.next() ){
				Integer death = (Integer)rs.getObject("death");
				if( (death==null && p.death==null) || death!=null && death.equals(p.death) ){
					return true;
				}
			}
			return false;
		} finally {
			s.close();
		}
	}

	private static final String selectByName =
 		"SELECT ismale, birth, death FROM Person WHERE name=?";

	private Set<Person> findByName( String name, Connection c ) throws SQLException {
		Set<Person> persons = new HashSet<Person>();
		PreparedStatement s = c.prepareStatement(selectByName);
		try {
			s.setString(1,name);
			ResultSet rs = s.executeQuery();
			while( rs.next() ){
				boolean ismale = rs.getBoolean("ismale");
				int birth = rs.getInt("birth");
				Integer death = (Integer)rs.getObject("death");
				persons.add( new Person(name,ismale,birth,death) );
			}
			return persons;
		} finally {
			s.close();
		}
	}

}

